from _typeshed import Incomplete
from amsdal_models.classes.model import Model as Model
from amsdal_models.querysets.errors import BulkOperationError as BulkOperationError, MultipleObjectsReturnedError as MultipleObjectsReturnedError, ObjectDoesNotExistError as ObjectDoesNotExistError
from amsdal_models.querysets.executor import DEFAULT_DB_ALIAS as DEFAULT_DB_ALIAS, Executor as Executor
from amsdal_utils.query.utils import Q
from typing import Any, Generic, TypeVar
from typing_extensions import Self

logger: Incomplete
ModelType = TypeVar('ModelType', bound='Model')

class QuerySetBase(Generic[ModelType]):
    """
    Base class for QuerySets.

    This class provides the base functionality for creating and manipulating
    query sets for model instances. It includes methods for filtering, ordering,
    and executing queries, as well as handling pagination and distinct results.

    """
    _entity: Incomplete
    _paginator: Incomplete
    _order_by: Incomplete
    _query_specifier: Incomplete
    _conditions: Incomplete
    _using: Incomplete
    def __init__(self, entity: type[ModelType]) -> None: ...
    @property
    def entity_name(self) -> str:
        """
        Get the name of the entity associated with the query set.

        This property returns the name of the model class associated with the query set.

        Returns:
            str: The name of the model class.
        """
    def using(self, value: str) -> Self:
        """
        Set the database alias to be used for the query.

        This method creates a copy of the current query set and sets the `_using`
        attribute to the specified database alias.

        Args:
            value (str): The database alias to be used for the query.

        Returns:
            Self: A new instance of the query set with the specified database alias.
        """
    @classmethod
    def _from_queryset(cls, queryset: QuerySetBase) -> Self: ...
    def _copy(self) -> Self: ...
    def __copy__(self) -> Self: ...
    def only(self, fields: list[str]) -> Self:
        """
        Limit the number of fields to be returned.

        This method creates a copy of the current query set and sets the `only`
        attribute to the specified fields, limiting the fields to be returned
        in the query results.

        Args:
            fields (list[str]): The fields to be returned.

        Returns:
            Self: A new instance of the query set with the specified fields.
        """
    def distinct(self, fields: list[str]) -> Self:
        """
        Return only distinct (different) values.

        This method creates a copy of the current query set and sets the `distinct`
        attribute to the specified fields, ensuring that only distinct values are
        returned in the query results.

        Args:
            fields (list[str]): The fields to be distinct.

        Returns:
            Self: A new instance of the query set with the specified distinct fields.
        """
    def filter(self, *args: Q, **kwargs: Any) -> Self:
        """
        Apply filters to the query. The filters are combined with AND.

        Args:
            args (Q): The filters to be applied.
            kwargs (Any): The filters to be applied.

        Returns:
            Self: A new instance of the query set with the applied filters.
        """
    def exclude(self, *args: Q, **kwargs: Any) -> Self:
        """
        Exclude filters from the query. The filters are combined with AND.

        Args:
            args (Q): The filters to be applied.
            kwargs (Any): The filters to be applied.

        Returns:
            Self: A new instance of the query set with the applied filters.
        """
    def _execute_query(self) -> list[dict[str, Any]]: ...
    def _execute_count(self) -> int: ...
    def _filter(self, *args: Q, negated: bool = ..., **kwargs: Any) -> Self: ...
    def order_by(self, *args: str) -> Self:
        """
        Order the query by the given fields.

        Args:
            args (str): The fields to order by.

        Returns:
            Self: A new instance of the query set with the specified order.
        """
    def __getitem__(self, index: slice | int) -> Self: ...
    def _create_instance(self, *, _is_partial: bool, **kwargs: Any) -> ModelType: ...
    def latest(self) -> Self:
        """
        Filter the query set to include only the latest version of the model instances.

        This method creates a copy of the current query set and applies a filter to
        include only the latest version of the model instances.

        Returns:
            Self: A new instance of the query set with the filter applied to include
            only the latest version of the model instances.
        """
    def _check_type(self, obj: ModelType) -> None: ...
    def bulk_update(self, objs: list[ModelType], using: str | None = ...) -> None:
        """
        Perform a bulk update on the given list of model instances.

        This method updates multiple instances of the model class defined in the
        `model` attribute of the manager in a single query. The `using` parameter
        allows specifying a database alias for the operation.

        Args:
            objs (list[ModelType]): A list of model instances to be updated.
            using (str | None): The database alias to be used for the bulk update operation.
            If None, the default database alias is used.

        Returns:
            None
        """
    def bulk_create(self, objs: list[ModelType], using: str | None = ...) -> None:
        """
        Perform a bulk creation of the given list of model instances.

        This method creates multiple instances of the model class defined in the
        `model` attribute of the manager in a single query. The `using` parameter
        allows specifying a database alias for the operation.

        Args:
            objs (list[ModelType]): A list of model instances to be created.
            using (str | None): The database alias to be used for the bulk creation operation.
            If None, the default database alias is used.

        Returns:
            None
        """
    def bulk_delete(self, objs: list[ModelType], using: str | None = ...) -> None:
        """
        Perform a bulk deletion of the given list of model instances.

        This method deletes multiple instances of the model class defined in the
        `model` attribute of the manager in a single query. The `using` parameter
        allows specifying a database alias for the operation.

        Args:
            objs (list[ModelType]): A list of model instances to be deleted.
            using (str | None): The database alias to be used for the bulk deletion operation.
            If None, the default database alias is used.

        Returns:
            None
        """

class QuerySet(QuerySetBase[ModelType], Generic[ModelType]):
    """
    Interface to access the database.

    This class provides methods to create and manipulate query sets for model instances.
    It includes methods for filtering, ordering, and executing queries, as well as handling
    pagination and distinct results.
    """
    def get(self, *args: Q, **kwargs: Any) -> QuerySetOneRequired[ModelType]:
        """
        Change the QuerySet to a QuerySetOneRequired. Query execution will return a single item or raise an error.

        Args:
            args (Q): The filters to be applied.
            kwargs (Any): The filters to be applied.

        Returns:
            QuerySetOneRequired: A new instance of the QuerySetOneRequired with the applied filters.
        """
    def get_or_none(self, *args: Q, **kwargs: Any) -> QuerySetOne[ModelType]:
        """
        Change the QuerySet to a QuerySetOne. Query execution will return a single item or None.

        Args:
            args (Q): The filters to be applied.
            kwargs (Any): The filters to be applied.

        Returns:
            QuerySetOne: A new instance of the QuerySetOne with the applied filters.
        """
    def first(self, *args: Q, **kwargs: Any) -> QuerySetOne[ModelType]:
        """
        Change the QuerySet to a QuerySetOne. Query execution will return the first item or None.

        Args:
            args (Q): The filters to be applied.
            kwargs (Any): The filters to be applied.

        Returns:
            QuerySetOneRequired: A new instance of the QuerySetOneRequired with the applied filters.
        """
    def count(self) -> QuerySetCount[ModelType]:
        """
        Change the QuerySet to a QuerySetCount. Query execution will return the count of items.

        Returns:
            QuerySetCount: A new instance of the QuerySetCount.
        """
    def execute(self) -> list[ModelType]:
        """
        Return the list of items.

        This method executes the query and returns a list of model instances.
        If the `only` attribute is set, partial model instances are created.

        Returns:
            list[ModelType]: A list of model instances.
        """
    def only(self, fields: list[str]) -> Self:
        """
        Limit the number of fields to be returned.

        This method creates a copy of the current query set and sets the `only`
        attribute to the specified fields, limiting the fields to be returned
        in the query results.

        Args:
            fields (list[str]): The fields to be returned.

        Returns:
            Self: A new instance of the query set with the specified fields.
        """
    def distinct(self, fields: list[str]) -> Self:
        """
        Return only distinct (different) values.

        This method creates a copy of the current query set and sets the `distinct`
        attribute to the specified fields, ensuring that only distinct values are
        returned in the query results.

        Args:
            fields (list[str]): The fields to be distinct.

        Returns:
            Self: A new instance of the query set with the specified distinct fields.
        """
    def filter(self, *args: Q, **kwargs: Any) -> Self:
        """
        Apply filters to the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to it. The filters can be provided as positional arguments or keyword
        arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied filters.
        """
    def exclude(self, *args: Q, **kwargs: Any) -> Self:
        """
        Exclude filters from the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to exclude certain results. The filters can be provided as positional
        arguments or keyword arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied exclusion filters.
        """
    def order_by(self, *args: str) -> Self:
        """
        Order the query by the given fields.

        This method creates a copy of the current query set and sets the `order_by`
        attribute to the specified fields, determining the order of the query results.

        Args:
            *args (str): The fields to order by.

        Returns:
            Self: A new instance of the query set with the specified order.
        """

class QuerySetOne(QuerySetBase[ModelType], Generic[ModelType]):
    """
    QuerySet class for models. QuerySet is executed to a single model object or None.

    This class provides methods to create and manipulate query sets for model instances.
    It includes methods for filtering, ordering, and executing queries, as well as handling
    pagination and distinct results. The query execution will return a single item or None.
    """
    _raise_on_multiple: bool
    def __init__(self, entity: type[ModelType]) -> None: ...
    def only(self, fields: list[str]) -> Self:
        """
        Limit the number of fields to be returned.

        This method creates a copy of the current query set and sets the `only`
        attribute to the specified fields, limiting the fields to be returned
        in the query results.

        Args:
            fields (list[str]): The fields to be returned.

        Returns:
            Self: A new instance of the query set with the specified fields.
        """
    def distinct(self, fields: list[str]) -> Self:
        """
        Return only distinct (different) values.

        This method creates a copy of the current query set and sets the `distinct`
        attribute to the specified fields, ensuring that only distinct values are
        returned in the query results.

        Args:
            fields (list[str]): The fields to be distinct.

        Returns:
            Self: A new instance of the query set with the specified distinct fields.
        """
    def filter(self, *args: Q, **kwargs: Any) -> Self:
        """
        Apply filters to the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to it. The filters can be provided as positional arguments or keyword
        arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied filters.
        """
    def exclude(self, *args: Q, **kwargs: Any) -> Self:
        """
        Exclude filters from the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to exclude certain results. The filters can be provided as positional
        arguments or keyword arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied exclusion filters.
        """
    def order_by(self, *args: str) -> Self:
        """
        Order the query by the given fields.

        This method creates a copy of the current query set and sets the `order_by`
        attribute to the specified fields, determining the order of the query results.

        Args:
            *args (str): The fields to order by.

        Returns:
            Self: A new instance of the query set with the specified order.
        """
    def execute(self) -> ModelType | None:
        """
        Query the database and return the single item or None.

        This method executes the query and returns a single model instance or None.
        If multiple items are found, a `MultipleObjectsReturnedError` is raised.

        Raises:
            MultipleObjectsReturnedError: If multiple items are found.

        Returns:
            ModelType | None: The single model instance or None if no items are found.
        """

class QuerySetOneRequired(QuerySetOne[ModelType], Generic[ModelType]):
    """
    QuerySet class for models. QuerySet is executed to a single model object or raises an error.

    This class provides methods to create and manipulate query sets for model instances.
    It includes methods for filtering, ordering, and executing queries, as well as handling
    pagination and distinct results. The query execution will return a single item or raise
    an error if no items are found.
    """
    def only(self, fields: list[str]) -> Self:
        """
        Limit the number of fields to be returned.

        This method creates a copy of the current query set and sets the `only`
        attribute to the specified fields, limiting the fields to be returned
        in the query results.

        Args:
            fields (list[str]): The fields to be returned.

        Returns:
            Self: A new instance of the query set with the specified fields.
        """
    def distinct(self, fields: list[str]) -> Self:
        """
        Return only distinct (different) values.

        This method creates a copy of the current query set and sets the `distinct`
        attribute to the specified fields, ensuring that only distinct values are
        returned in the query results.

        Args:
            fields (list[str]): The fields to be distinct.

        Returns:
            Self: A new instance of the query set with the specified distinct fields.
        """
    def filter(self, *args: Q, **kwargs: Any) -> Self:
        """
        Apply filters to the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to it. The filters can be provided as positional arguments or keyword
        arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied filters.
        """
    def exclude(self, *args: Q, **kwargs: Any) -> Self:
        """
        Exclude filters from the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to exclude certain results. The filters can be provided as positional
        arguments or keyword arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied exclusion filters.
        """
    def order_by(self, *args: str) -> Self:
        """
        Order the query by the given fields.

        This method creates a copy of the current query set and sets the `order_by`
        attribute to the specified fields, determining the order of the query results.

        Args:
            *args (str): The fields to order by.

        Returns:
            Self: A new instance of the query set with the specified order.
        """
    def execute(self) -> ModelType:
        """
        Return the single item.

        This method executes the query and returns a single model instance.
        If no items are found, an `ObjectDoesNotExistError` is raised.

        Raises:
            ObjectDoesNotExistError: If no items are found.

        Returns:
            ModelType: The single model instance.
        """

class QuerySetCount(QuerySetBase[ModelType], Generic[ModelType]):
    """
    QuerySet class for models. QuerySet is executed to a count of items.

    This class provides methods to create and manipulate query sets for model instances.
    It includes methods for filtering, ordering, and executing queries, as well as handling
    pagination and distinct results. The query execution will return the count of items.

    """
    def only(self, fields: list[str]) -> Self:
        """
        Limit the number of fields to be returned.

        This method creates a copy of the current query set and sets the `only`
        attribute to the specified fields, limiting the fields to be returned
        in the query results.

        Args:
            fields (list[str]): The fields to be returned.

        Returns:
            Self: A new instance of the query set with the specified fields.
        """
    def distinct(self, fields: list[str]) -> Self:
        """
        Return only distinct (different) values.

        This method creates a copy of the current query set and sets the `distinct`
        attribute to the specified fields, ensuring that only distinct values are
        returned in the query results.

        Args:
            fields (list[str]): The fields to be distinct.

        Returns:
            Self: A new instance of the query set with the specified distinct fields.
        """
    def filter(self, *args: Q, **kwargs: Any) -> Self:
        """
        Apply filters to the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to it. The filters can be provided as positional arguments or keyword
        arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied filters.
        """
    def exclude(self, *args: Q, **kwargs: Any) -> Self:
        """
        Exclude filters from the query. The filters are combined with AND.

        This method creates a copy of the current query set and applies the specified
        filters to exclude certain results. The filters can be provided as positional
        arguments or keyword arguments, and they are combined using the AND operator.

        Args:
            *args (Q): The filters to be applied as positional arguments.
            **kwargs (Any): The filters to be applied as keyword arguments.

        Returns:
            Self: A new instance of the query set with the applied exclusion filters.
        """
    def order_by(self, *args: str) -> Self:
        """
        Order the query by the given fields.

        This method creates a copy of the current query set and sets the `order_by`
        attribute to the specified fields, determining the order of the query results.

        Args:
            *args (str): The fields to order by.

        Returns:
            Self: A new instance of the query set with the specified order.
        """
    def execute(self) -> int:
        """
        Return the count of items.

        This method executes the query and returns the count of model instances
        that match the query criteria.

        Returns:
            int: The count of model instances.
        """
